@@ -21,10 +21,15 @@
 __author__ = """Peter Rowlands"""
 __email__ = "peter@pmrowla.com"
 __license__ = "GPLv3"
-__version__ = "0.3.2"
+__version__ = "1.0.0"
 
 
+from loguru import logger
+
 from .archive import LMArchive, LMArchiveInfo, LMCompressType
 from .lsb import LMScript, LNSCompiler, LNSDecompiler
 
 __all__ = ["LMArchive", "LMArchiveInfo", "LMCompressType", "LMScript", "LNSCompiler", "LNSDecompiler"]
+
+
+logger.disable("livemaker")
