@@ -1,6 +1,6 @@
 # -*- coding: utf-8
 #
-# Copyright (C) 2019 Peter Rowlands <peter@pmrowla.com>
+# Copyright (C) 2020 Peter Rowlands <peter@pmrowla.com>
 # Copyright (C) 2014 tinfoil <https://bitbucket.org/tinfoil/>
 #
 # This file is a part of pylivemaker.
@@ -19,17 +19,14 @@
 """Core lmscript classes."""
 
 import enum
-import logging
 from abc import ABC, abstractmethod
 
 import construct
 import numpy
+from loguru import logger
 from lxml import etree
 
 
-log = logging.getLogger(__name__)
-
-
 class BaseSerializable(ABC):
     """Base class for serializable LiveMaker objects.
 
@@ -473,7 +470,7 @@ class Param(BaseSerializable):
     def to_xml(self):
         xml = self.to_lsc()
         if self.type == ParamType.Var and "\x01" in xml:
-            log.warn('Replacing invalid xml char "\\x01" in varname {}'.format(self.value))
+            logger.warning('Replacing invalid xml char "\\x01" in varname {}'.format(self.value))
             xml = xml.replace("\x01", "*")
         return xml
 
@@ -816,7 +813,7 @@ class LiveParser(BaseSerializable):
     def to_xml(self):
         xml = self.to_lsc()
         if "\x01" in xml:
-            log.warn('Replacing invalid xml char "\\x01"')
+            logger.warning('Replacing invalid xml char "\\x01"')
             xml = xml.replace("\x01", "*")
         return xml
 
@@ -862,7 +859,7 @@ class LiveParser(BaseSerializable):
                     return _resolve("____arg", exprs)
                 return "{} = {}".format(e.name, _resolve(e.name, exprs))
             else:
-                log.warn("Last entry in LiveParser was not a To statement: {}".format(self.entries[-1]))
+                logger.warning("Last entry in LiveParser was not a To statement: {}".format(self.entries[-1]))
         return ""
 
 
