@@ -19,22 +19,20 @@
 """LiveMaker LiveNovel LNS script classes."""
 
 import enum
-import logging
 import os
 import re
 import _markupbase
 from bisect import bisect
-from hashlib import sha256
 
 import construct
 
+from loguru import logger
+
 from lxml import etree
 
 from .core import BaseSerializable, LiveParser
-from ..exceptions import BadLnsError
-
-
-log = logging.getLogger(__name__)
+from ..exceptions import BadLnsError, InvalidCharError
+from .translate import BaseTranslatable
 
 
 class LNSTag(enum.Enum):
@@ -255,7 +253,7 @@ class TWdChar(BaseTWdReal):
         try:
             ch.encode("cp932")
         except UnicodeEncodeError:
-            raise BadLnsError("'{}' is not a valid CP932 character".format(ch))
+            raise InvalidCharError(ch)
         self._keys.update(("ch", "decorator"))
         self.ch = ch
         self.decorator = decorator
@@ -398,6 +396,25 @@ class TWdOpeEvent(BaseTWdGlyph):
             return "{{{0}{1}}}".format(e, args)
         return LNSTag.open(LNSTag.event, {"VALUE": "\\r\\n".join([e] + args)})
 
+    @property
+    def _parts(self):
+        return self.event.split("\r\n")
+
+    @property
+    def name(self):
+        name = self._parts[0]
+        if self.is_system(name):
+            return name[1:]
+        return name
+
+    @property
+    def args(self):
+        return self.parts[1:]
+
+    @staticmethod
+    def is_system(name):
+        return name.startswith("\x01")
+
 
 class TWdOpeVar(BaseTWdGlyph):
     """Insert the value of the specified variable.
@@ -468,7 +485,7 @@ class TWdImg(BaseTWdReal):
 
     Note:
         This displays the image inline with text, and is not the same thing as displaying a CG.
-        One use case is to insert icons for replaying sounds into the history backlog.
+        One use case is to insert icons for replaying sounds into the history backlogger.
 
     """
 
@@ -607,7 +624,7 @@ class TWdCondition(BaseSerializable):
     """Text display conditions.
 
     Display condition for a glyph determines things like whether or not it will
-    be only displayed the history backlog.
+    be only displayed the history backlogger.
 
     Args:
         count: The total number of TWd glyphs affected by this condition.
@@ -765,7 +782,7 @@ class TpWord(BaseSerializable):
         dec = LNSDecompiler()
         xml = dec.decompile(self)
         if "\x01" in xml:
-            log.warn("Removing illegal xml char \\x01")
+            logger.warning("Removing illegal xml char \\x01")
             xml = xml.replace("\x01", "*")
         return etree.CDATA(xml)
 
@@ -877,7 +894,7 @@ class TpWord(BaseSerializable):
             blocks (:class:`LNSText`): Replacement blocks. ``blocks`` should be an object
                 previously returned by `get_text_blocks()` (but with modified text).
             strict (bool): If True, `BadLnsError` will be raised if ``blocks`` contains blocks
-                with SHA256 digests which do not match the current TpWord.
+                with Blake2 digests which do not match the current TpWord.
         """
         new_body = self.body[:]
         if strict and blocks != self.get_text_blocks():
@@ -1232,7 +1249,7 @@ class LNSCompiler(_markupbase.ParserBase):
 
         """
         if not script.startswith(";pylm"):
-            log.warn("Attempting to compile script which was not generated by pylivemaker.")
+            logger.warning("Attempting to compile script which was not generated by pylivemaker.")
         for line in script.splitlines():
             if not line.startswith(";"):
                 self.feed(line)
@@ -1286,7 +1303,7 @@ class LNSCompiler(_markupbase.ParserBase):
                         self.handle_eventtag("", [])
                         k = i + 2
                     else:
-                        log.warn("got < or { at end of line")
+                        logger.warning("got < or { at end of line")
                         self.handle_data("<")
                         k = i + 1
                 else:
@@ -1517,12 +1534,12 @@ class LNSCompiler(_markupbase.ParserBase):
             if self.version and self.version < 102:
                 # TODO for LNS ver < 102 we have to convert var name back into
                 # a live parser
-                log.warn("Compiling script ver < 102 with VAR tags not fully supported")
+                logger.warning("Compiling script ver < 102 with VAR tags not fully supported")
             d["var_name"] = attrs.get("NAME", "")
             d["unk3"] = int(attrs.get("unk3", 0))
             self.tpword_body.append(TWdOpeVar(**d))
         else:
-            log.warn("Unexpected tag: {}".format(tag))
+            logger.warning("Unexpected tag: {}".format(tag))
 
     def handle_endtag(self, tag):
         tag = LNSTag[tag.lower()]
@@ -1551,7 +1568,7 @@ class LNSCompiler(_markupbase.ParserBase):
         return s
 
 
-class LNSTextBlock:
+class LNSTextBlock(BaseTranslatable):
     """Contiguous text block in a TpWord body.
 
     Args:
@@ -1559,6 +1576,7 @@ class LNSTextBlock:
         start (int): TpWord body index of the first TWdChar in this line
         end (int): TpWord body index of the first TWdGlyph following this line.
             If `end` is None, it will be set to ``start + len(text)``.
+        name_label (str): associated namelabel event (speaker name).
 
     Text blocks are defined as continuous runs of `TWdChar` and `<BR>` line-breaks
     (`TWdOpeReturn` with `break_type == BreakType.LINE`).
@@ -1574,8 +1592,9 @@ class LNSTextBlock:
         To test string equality between, compare the ``text`` attributes.
     """
 
-    def __init__(self, text, start, end=None):
-        self.text = text
+    def __init__(self, text, start, end=None, name_label=None):
+        super().__init__(text)
+        self.name_label = name_label
         self._start = start
         if self._start < 0:
             raise ValueError("LNSTextLine start must be >= 0")
@@ -1585,7 +1604,6 @@ class LNSTextBlock:
             self._end = end
         if self._end <= start:
             raise ValueError("LNSTextLine end must be > start")
-        self._digest = self.text_digest(self.text)
 
     @property
     def start(self):
@@ -1595,28 +1613,9 @@ class LNSTextBlock:
     def end(self):
         return self._end
 
-    @property
-    def digest(self):
-        return self._digest
-
-    @property
-    def text(self):
-        return "\n".join(self._text)
-
-    @text.setter
-    def text(self, text):
-        self._text = text.splitlines()
-
-    @property
-    def lines(self):
-        return self._text
-
     def __hash__(self):
         return hash((self.start, self.end, self.digest))
 
-    def __eq__(self, other):
-        return hash(self) == hash(other)
-
     def __lt__(self, other):
         return self.start < other.start and self.end < other.end
 
@@ -1625,15 +1624,6 @@ class LNSTextBlock:
             self.end > other.start and self.end <= other.end
         )
 
-    def __str__(self):
-        return str(self.text)
-
-    @staticmethod
-    def text_digest(text):
-        hash_ = sha256()
-        hash_.update(text.encode("utf-8"))
-        return hash_.hexdigest()[16]
-
 
 class LNSText:
     """Convenience container for accessing text blocks in a TpWord body."""
@@ -1684,6 +1674,7 @@ class LNSText:
         """
         blocks = LNSText()
         cur_block = []
+        cur_name = None
         start = 0
         for i, w in enumerate(tpword.body):
             block_break = True
@@ -1696,6 +1687,12 @@ class LNSText:
                 if cur_block and w.break_type == BreakType.LINE:
                     cur_block.append("\n")
                     block_break = False
+            elif w.type == TWdType.TWdOpeEvent:
+                if w.name == "NAMELABEL":
+                    if w.is_system:
+                        cur_name = None
+                    else:
+                        cur_name = w.args[0]
 
             if block_break:
                 if cur_block:
@@ -1703,6 +1700,6 @@ class LNSText:
                     # create blank message box screens, we can strip them here
                     # (they will still be preserved in-game)
                     text = "".join(cur_block).rstrip("\n")
-                    blocks.add(LNSTextBlock(text, start))
+                    blocks.add(LNSTextBlock(text, start, name_label=cur_name))
                 cur_block.clear()
         return blocks
